<?php
namespace FM\Service;

use FM\Factory\SolicitacaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SolicitacaoService {
    
    protected $SolicitacaoFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    
    public function __construct (SolicitacaoFactory $SolicitacaoFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager) {
        $this->SolicitacaoFactory           = $SolicitacaoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection(); 
    }
    
    // Busca as solicitações
    public function getRequests(){
        $requestsList = $this->SolicitacaoFactory->getRequests();
        return $requestsList;
    }
    
    // Busca os tipos de solicitação
    public function getRequestTypes(){
        $typesList = $this->SolicitacaoFactory->getRequestTypes();
        return $typesList;
    }
    
   // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestInfo($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca o histórico de uma solicitação
    public function getRequestHistoric($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestHistoric($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca todas as hierarquias
    public function insertRequestType($params){
        $nrtiposolicitacao  = $this->newCode->getCodeByTableName('FCM_TIPOSOLICITACAO', 20, 1, 10, true);
        
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $nrtiposolicitacao,
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR'],
            'NRORG' => 20
        );

        try {
            $this->SolicitacaoFactory->insertRequestType($typeParams);
            $response = "Inserido com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
    
    // Processo de clasificação -> Alteração nas tabelas de solicitação/sol_tarefas, checklistAgend e  histórico
    public function classifyRequest($params){
 
        $requestParams = array(
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NMSOLICITACAO' => $params['NMSOLICITACAO'],
            'IDSITUACAO' => '2',
            'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
            'IDPRIORIDADE' => $params['IDPRIORIDADE'],
            'NRFUNCRESPONSAVEL' => $params['NRFUNCRESPONSAVEL'],
            'DSSOLICITACAO' => $params['DSSOLICITACAO']
        );
 
        $nrChecklistAgend  = $this->newCode->getCodeByTableName('FCM_CHECKLISTAGEND', 20, 1, 10, true);       
        $checklistParams = array(
            'NRCHECKLISTAGEND' => $nrChecklistAgend,
            'NRLOCALCLIENTE' => $params['NRLOCALCLIENTE'],
            'NRCHECKLIST' => $params['NRCHECKLIST'],
            'NRFUNCIONARIO' => $params['NRFUNCRESPONSAVEL'],
            'NRORG' => 20,
            'NMCHECKLISTAGEND' => $params['NMSOLICITACAO']
        );
        
        $nrSolicitacaoTarefas  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOTAREFAS', 20, 1, 10, true);    
        $taskRequestParams = array(
            'NRSOLICITACAOTAREFAS' => $nrSolicitacaoTarefas,
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NRCHECKLISTAGEND' => $nrChecklistAgend,
            'NRORG' => 20
        );
        
        $nrSolicitacaoHistorico  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOHISTORICO', 20, 1, 10, true);    
        $historicParams = array(
            'NRSOLICITACAOHISTORICO' => $nrSolicitacaoHistorico,
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NMSOLICITACAO' => $params['NMSOLICITACAO'],
            'DTHISTSOLICITACAO' => $params['DTHISTSOLICITACAO'],
            'IDSITUACAO' => '2',
            'IDTIPOHISTORICO' => '2',
            'NRFUNCSOLICITANTE' => $params['NRFUNCRESPONSAVEL'],
            'DSSOLICITACAO' => $params['DSSOLICITACAO'],
            'NRORG' => 20,
        );
        // debugApi([$historicParams]);

        $this->connection->beginTransaction();
        try {
            $this->SolicitacaoFactory->updateRequest($requestParams);
            $this->SolicitacaoFactory->createTask($checklistParams); // Cria uma tarefa
            $this->SolicitacaoFactory->createTaskRequest($taskRequestParams); // Cria uma relação de tarefa com solicitação
            $this->SolicitacaoFactory->insertHistoricData($historicParams);
            
            $this->connection->commit();
            $response = "Classificado com sucesso";
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response = $e->getMessage();
        }

        return $response;
    }
    
     // Edita o tipo de solicitação
    public function updateRequestType($params){
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR']
        );
        
        try {
            $this->SolicitacaoFactory->updateRequestType($typeParams);
            $response = "Editado com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
}
?>