<?php

namespace Teknisa\Libs\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Service\Api;

class RequestListenerOAuth extends Listener {

    const OAUTH_HEADER = "OAuth-Token";
    const OAUTH_SECRET_ID = "OAuth-SecretId";
    const OAUTH_KEEP_CONNECTED = "OAuth-KeepConnected";
    const KEEP_CONNECTED_POSITIVE = "Yes";

    private $httpKernel;
    private $oauth;
    private $sessionManager;
    private $apiService;

    public function __construct(Kernel $kernel, OAuthCustom $oauth, Session $sessionManager, Api $apiService) {
        $this->httpKernel = $kernel;
        $this->oauth = $oauth;
        $this->sessionManager = $sessionManager;
        $this->apiService = $apiService;
    }

    private function readPublicRoutes(){
        $urls = file_get_contents(__DIR__ . "/../../../../config/publicRoutes.json");
        $publicRoutes = json_decode($urls, true);
        $productUrls = @file_get_contents(Utilities::getProductBasePath(true) . "/backend/config/publicRoutes.json");
        if(!empty($productUrls)) {
            $publicRoutes = array_merge($publicRoutes, json_decode($productUrls, true));
        }
        return $publicRoutes;
    }

    public function preDispatch(Request $request)
    {        
        try {
            $this->apiService->setLastRequestTime($request->getRoutePath());
            $token = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_HEADER);
            if (!is_null($token)) {
                $this->checkAccessAndSetSession($token);
            }
            $this->checkPublicRoute($request);
        } catch (\Exception $e){
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }

    /**
     * @param $token
     */
    private function checkAccessAndSetSession($token)
    {
        $secretId = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_SECRET_ID);
        $keepConn = $this->httpKernel->getHttpRequest()->getHeaders()->get(self::OAUTH_KEEP_CONNECTED);
        $keepConn = $keepConn == self::KEEP_CONNECTED_POSITIVE ? true : false;
        $access   = $this->oauth->checkAccess($token, $secretId, array(), $keepConn);

        if (!empty($access)) {
            if($this->sessionManager->getId() != $access['sessionId']) {
                $this->sessionManager->start();
                $this->sessionManager->destroy();
                $this->sessionManager->setId($access['sessionId']);
            }
            $this->sessionManager->start();

            if(!$this->sessionManager->has("NRORG")) {
                $this->sessionManager->set('NRORG', $access['service']->getNrOrg());
                $this->sessionManager->set('CDOPERADOR', $access['service']->getCdOperator());
            }
        }
    }

    /**
     * @param Request $request
     * @throws \Exception
     */
    private function checkPublicRoute(Request $request)
    {
        if (!in_array($request->getRoutePath(), $this->readPublicRoutes()) && !$this->sessionManager->has("NRORG")) {
            throw new \Exception("Access denied.");
        }
    }
}