import {
  CollapseCard, Select, Text, Date as DateInput, Row, AlertService, IComponentRender,
} from '@zeedhi/common';
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class FCMAnaliseNaoConformidade {
  public Gestores: IDictionary = {};

  public loaded: boolean = false;

  public LocalType: Array<IDictionary> = [];

  public Local: Array<IDictionary> = [];

  public Sector: Array<IDictionary> = [];

  constructor() {
    this.requestGestores('/NaoConformidades/Gestor')
      .then((gestores: any) => {
        this.Gestores = gestores;
      });
  }

  public setTypeLocal({ component }:{ component:Select }) {
    Http.get('/getLocalType').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.options.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMTIPOLOCALCLI,
          value: element.NRTIPOLOCALCLI,
        });
      });
    });
  }

  public setLocal({ component }:{ component:Select }) {
    Http.get('/getLocal').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.local.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMLOCALCLIENTE,
          value: element.NRLOCALCLIENTE,
        });
      });
    });
  }

  public setSector({ component }:{ component:Select }) {
    Http.get('/getSector').then((response: IDictionary) => {
      component.datasource.data = [];
      response.data.dataset.options.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMSETOR,
          value: element.NRSETOR,
        });
      });
    });
  }

  public getAllData() {
    let dataDe = Metadata.getInstance<DateInput>('DataInicioPeriodo').value;
    let dataAte = Metadata.getInstance<DateInput>('DataFimPeriodo').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    const test = Metadata.getInstance<Select>('SelectLocal').value;
    return {
      LocalType: Metadata.getInstance<Select>('SelectTipoLocal').value,
      Local: test,
      Sector: Metadata.getInstance<Select>('SelectSetor').value,
      PeriodoDe: dataDe,
      PeriodoAte: dataAte,
    };
  }

  public getAllDataFilters(type: string) {
    let dateBegin = Metadata.getInstance<DateInput>(`DATEBEGIN_${type}`).value;
    let dateEnd = Metadata.getInstance<DateInput>(`DATEEND_${type}`).value;
    if (dateBegin) {
      dateBegin = (dateBegin.length === 10) ? dateBegin : null;
    } else {
      dateBegin = null;
    }
    if (dateEnd) {
      dateEnd = (dateEnd.length === 10) ? dateEnd : null;
    } else {
      dateEnd = null;
    }
    const nrLocalCliente = Metadata.getInstance<Select>(`NRLOCALCLIENTE_${type}`).value;
    const nrTipoLocalCli = Metadata.getInstance<Select>(`NRTIPOLOCALCLI_${type}`).value;
    const nrSetor = Metadata.getInstance<Select>(`NRSETOR_${type}`).value;
    let strFilters = '';
    if (nrLocalCliente) {
      strFilters += `NRLOCALCLIENTE=${nrLocalCliente}&`;
    }
    if (nrTipoLocalCli) {
      strFilters += `NRTIPOLOCALCLI=${nrTipoLocalCli}&`;
    }
    if (nrSetor) {
      strFilters += `NRSETOR=${nrSetor}&`;
    }
    if (dateBegin) {
      strFilters += `DATEBEGIN=${dateBegin}&`;
    }
    if (dateEnd) {
      strFilters += `DATEEND=${dateEnd}&`;
    }
    return strFilters;
  }

  public getDataCards() {
    Http.post('/getAnaliseCriterio', this.getAllData()).then((response: IDictionary) => {
      const dados = response.data.dataset.criteriosConformidade;
      const iterable = Metadata.getInstance<IterableComponentRender>('iterable_CardsDeLocal');
      iterable.reload();
      iterable.datasource.data = [];
      Object.keys(dados).forEach((element: string) => {
        iterable.datasource.data.push({
          Local: element,
          Conformidades: `${dados[element].length}`,
          dadosFilho: dados[element],
        });
      });
    });
  }

  public filterPeriodo() {
    let dataDe = Metadata.getInstance<DateInput>('DataInicioPeriodoCriterio').value;
    let dataAte = Metadata.getInstance<DateInput>('DataFimPeriodoCriterio').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    return {
      PeriodoDe: dataDe,
      PeriodoAte: dataAte,
    };
  }

  public principaisNaoConformidade() {
    const iterable = Metadata.getInstance<IterableComponentRender>('iterable_Cards');
    iterable.datasource.data = [];
    Http.post('/getPrincipaisCriterio', this.filterPeriodo()).then((response: IDictionary) => {
      response.data.dataset.principaisConformidade.forEach((element: IDictionary) => {
        iterable.datasource.data.push({
          tipo: element.NMCRITERIOTARGERCHK,
          quantidade: element.QTDADE,
        });
      });
    });
  }

  public expandCard({ component }:{ component:Text }) {
    component.text = (component.text === 'ver mais ') ? 'ver menos' : 'ver mais ';
    const nomeComponent = component.name.split('_')[1];
    const card = Metadata.getInstance<CollapseCard>(`Cards_${nomeComponent}`);
    card.expanded = !card.expanded;
  }

  public getDateFather({ component }:{ component:IterableComponentRender }) {
    const iterable = Metadata.getInstance<IterableComponentRender>('iterable_CardsDeLocal');
    const nomeComponent = component.name.split('_')[1];
    const valores = iterable.datasource.data.filter((dados: IDictionary) => dados.Local === nomeComponent);
    valores[0].dadosFilho.forEach((element: IDictionary) => {
      component.datasource.data.push({
        conformidades: element,
      });
    });
  }

  public funcionariosNaoConformidades() {
    const strFilters: string = this.getAllDataFilters('FUNC');
    const nConformidadesGestor: string = `/NaoConformidades/Gestor?${strFilters}`;
    const rowComponent: Row = Metadata.getInstance('LinhaComIterableFUNC');
    rowComponent.children = [] as IComponentRender[];
    this.requestGestores(nConformidadesGestor)
      .then((gestores: any) => {
        this.Gestores = gestores;
        // for (const nrGestor in this.Gestores) {
        Object.keys(this.Gestores).forEach((nrGestor) => {
          // for (const nrFunc in this.Gestores[nrGestor].FUNCIONARIOS) {
          Object.keys(this.Gestores[nrGestor].FUNCIONARIOS).forEach((nrFunc) => {
            const columnItem = this.createColumnItem(nrFunc);
            const cardItem = this.createCardItem(nrFunc, this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].NMFUNCIONARIO, this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].TOTALNAOCONFORMIDADES, 'FUNC');
            const separator = this.createSeparator(nrFunc);
            cardItem?.children?.push(separator);
            const results: IComponentRender = this.createTagResults();
            this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].NAOCONFORMIDADES.forEach((naoConformidadeITem: string, index: number) => {
              const naoConformidade = this.createNaoConformidade(index.toString(), naoConformidadeITem);
              results.children?.push(naoConformidade);
            });
            cardItem.children?.push(results);
            columnItem.children?.push(cardItem);
            rowComponent.children.push(columnItem);
          });
          // }
        });
        // }
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          top: true,
          left: false,
          color: 'error',
          text: err,
          timeout: 3000,
        });
      });
  }

  public createSeparator(nrFunc: string): IComponentRender {
    const separator: IComponentRender = {
      name: `basicUsageDivider_${nrFunc}`,
      component: 'ZdDivider',
      cssClass: 'zd-mb-5',
    };
    return separator;
  }

  public createCardItem(nrFunc: string, nmCard: string, numConformidades: string, typeCard: string): IComponentRender {
    const cardItem: IComponentRender = {
      name: `Cards_${nrFunc}${typeCard}`,
      component: 'ZdCollapseCard',
      readonly: true,
      events: {
        // onCreated: '{{FCMAnaliseNaoConformidade.gestoresIterableConformidades}}'
        // onCreated: this.gestoresIterableConformidades.bind(this),
      },
      expandButton: {
        name: `primary_${nrFunc}${typeCard}`,
        component: 'ZdText',
        text: 'ver mais ',
        cssClass: 'botaoVerMais',
        events: {
          click: this.expandCard.bind(this),
        },
      },
      header: [
        {
          name: 'RowCard',
          component: 'ZdRow',
          children: [
            {
              name: 'ColunaInternaCardComLocal',
              component: 'ZdCol',
              cols: 3,
              children: [
                {
                  name: 'RowCard',
                  component: 'ZdRow',
                  align: 'center',
                  children: [
                    {
                      name: 'IconeMap',
                      component: 'ZdIcon',
                      cssClass: 'iconStyle iconStyleSpace',
                      iconName: 'mdi-map-marker-outline',
                    },
                    {
                      name: 'TituloCardLocal',
                      component: 'ZdText',
                      text: `${nmCard}`,
                      cssClass: 'TitleCard',
                    },
                  ],
                },
              ],
            },
            {
              name: 'ColunaInternaCardComNaoConformidades',
              component: 'ZdCol',
              cols: 9,
              children: [
                {
                  name: 'RowCard',
                  component: 'ZdRow',
                  align: 'center',
                  children: [
                    {
                      name: 'IconeInfo',
                      component: 'ZdIcon',
                      cssClass: 'iconStyle iconStyleSpace',
                      iconName: 'mdi-alert-outline',
                      dense: true,
                    },
                    {
                      name: 'InfosDentroDoCard',
                      component: 'ZdText',
                      text: `${numConformidades} critérios não conformes`,
                      cssClass: 'SubTitleCard',
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
      children: [
      ],
    };
    return cardItem;
  }

  public createTagResults(): IComponentRender {
    const tagR: IComponentRender = {
      name: 'tagResult',
      component: 'ZdTag',
      tag: 'ul',
      children: [],
    };
    return tagR;
  }

  public createNaoConformidade(index: string, criterioTarefa: string): IComponentRender {
    const naoConformidade: IComponentRender = {
      name: `item-${index}`,
      component: 'ZdText',
      text: `${criterioTarefa}<br />`,
      cssClass: 'dot',
      tag: 'li',
    };
    return naoConformidade;
  }

  public createColumnItem(nrFunc: string): IComponentRender {
    const columnItem: IComponentRender = {
      name: `ColunaComCard_${nrFunc}`,
      component: 'ZdCol',
      cssClass: 'zd-px-2 zd-py-0 zd-pb-3',
      children: [
      ],
    };
    return columnItem;
  }

  public gestoresIterableConformidades({ component }: { component: CollapseCard }) {
    const nrGestor = component.name.split('_')[1];
    // for (let nrFunc in this.Gestores[nrGestor].FUNCIONARIOS) {
    Object.keys(this.Gestores[nrGestor].FUNCIONARIOS).forEach((nrFunc) => {
      component.children.push(this.createSeparator(nrFunc));
      const nameFunc: string = this.Gestores[nrGestor].FUNCIONARIOS[nrFunc][0].NMFUNCIONARIO;
      const textFunc = {
        name: nameFunc,
        component: 'ZdText',
        text: nameFunc,
      };
      component.children.push(textFunc);
      const results: IComponentRender = {
        name: 'tagResult',
        component: 'ZdTag',
        tag: 'ul',
        children: [],
      };
      this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].forEach((funcionario: IDictionary, index: number) => {
        const naoConformidade: IComponentRender = {
          name: `item-${index.toString()}`,
          component: 'ZdText',
          text: `${funcionario.DSCRITERIOTAREFA}<br />`,
          cssClass: 'dot',
          tag: 'li',
        };
        results?.children?.push(naoConformidade);
      });
      component.children.push(results);
    });
    // }
  }

  public requestGestores(url: string) {
    return new Promise((resolve, reject) => {
      // const nConformidadesGestor = '/NaoConformidades/Gestor?' + strFilters;
      Http.get(url)
        .then((response) => {
          const promiseGestores: Array<Promise<any>> = [];
          const data: Array<IDictionary> = response.data.dataset.conformidadesGestor;
          const gestores: IDictionary = {};
          const auxObject: IDictionary = {};
          data.forEach((item: IDictionary) => {
            // checa se o funcionario tem um func pai válido. se não tiver, o item é ignorado
            if (item.NRFUNCIONARIOPAI !== null) {
              const nrfuncPaiString = item.NRFUNCIONARIOPAI.toString();
              // checa se o gestor já foi inserido na lista de gestores
              // se não estiver na lista, vai ser inserido e criado as propriedades
              if (!(nrfuncPaiString in gestores)) {
                gestores[nrfuncPaiString] = {};
                gestores[nrfuncPaiString].TOTALNAOCONFORMIDADES = 0;
                gestores[nrfuncPaiString].FUNCIONARIOS = {};
                // criando o array de promises
                promiseGestores.push(Http.get(`/Employee?NRFUNCIONARIO=${nrfuncPaiString}`));
              }
              const nrFuncStr = item.NRFUNCIONARIO.toString();
              if (!(nrFuncStr in gestores[nrfuncPaiString].FUNCIONARIOS)) {
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr] = {};
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NMFUNCIONARIO = item.NMFUNCIONARIO;
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NAOCONFORMIDADES = [];
                auxObject[nrFuncStr] = {};
              }
              const criterioTarefa = item.DSCRITERIOTAREFA;
              if (!(criterioTarefa in auxObject[nrFuncStr])) {
                auxObject[nrFuncStr][criterioTarefa] = 1;
              } else {
                auxObject[nrFuncStr][criterioTarefa] += 1;
              }
              gestores[nrfuncPaiString].TOTALNAOCONFORMIDADES += 1;
              // gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NAOCONFORMIDADES.push(item.DSCRITERIOTAREFA);
            }
          });
          Object.keys(gestores).forEach((nrGestor) => {
            Object.keys(gestores[nrGestor].FUNCIONARIOS).forEach((nrFunc) => {
              let totalNConformidades = 0;
              Object.keys(auxObject[nrFunc]).forEach((nConformidade) => {
                gestores[nrGestor].FUNCIONARIOS[nrFunc].NAOCONFORMIDADES.push(`${nConformidade} (${auxObject[nrFunc][nConformidade]}x)`);
                totalNConformidades += auxObject[nrFunc][nConformidade];
              });
              gestores[nrGestor].FUNCIONARIOS[nrFunc].TOTALNAOCONFORMIDADES = totalNConformidades;
            });
          });
          // garantindo que vai executar de forma sincrona
          Promise.all(promiseGestores)
            .then((respPromises: Array<IDictionary>) => {
              respPromises.forEach((responses: IDictionary) => {
                const gestor = responses.data.dataset.employee[0];
                const nrFuncString = gestor.NRFUNCIONARIO.toString();
                gestores[nrFuncString].NMGESTOR = gestor.NMFUNCIONARIO;
                // gestores[nrFuncString].NRFUNCIONARIO = gestor.NRFUNCIONARIO;
              });
              resolve(gestores);
            })
            .catch((err) => {
              AlertService.show({
                name: 'basic-alert',
                top: true,
                left: false,
                color: 'error',
                text: err,
                timeout: 3000,
              });
              reject(err);
            });
        })
        .catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            top: true,
            left: false,
            color: 'error',
            text: err,
            timeout: 3000,
          });
          reject(err);
        });
    });
  }

  public childrensConformidades() {
    const strFilters = this.getAllDataFilters('GESTOR');
    const nConformidadesGestor = `/NaoConformidades/Gestor?${strFilters}`;
    const rowComponent: Row = Metadata.getInstance('LinhaComIterableGESTOR');
    rowComponent.children = [] as IComponentRender[];
    this.requestGestores(nConformidadesGestor)
      .then((gestores: any) => {
        this.Gestores = gestores;
        Object.keys(this.Gestores).forEach((nrFuncGestor) => {
          const nmGestor: string = this.Gestores[nrFuncGestor].NMGESTOR;
          const numConformidades: string = this.Gestores[nrFuncGestor].TOTALNAOCONFORMIDADES.toString();
          const columnItem = this.createColumnItem(nrFuncGestor);
          const cardItem = this.createCardItem(nrFuncGestor, nmGestor, numConformidades, 'GESTOR');
          const separator = this.createSeparator(nrFuncGestor);
          cardItem?.children?.push(separator);
          Object.keys(this.Gestores[nrFuncGestor].FUNCIONARIOS).forEach((nrFunc) => {
            const nameFunc: string = this.Gestores[nrFuncGestor].FUNCIONARIOS[nrFunc].NMFUNCIONARIO;
            const textFunc = {
              name: nameFunc,
              component: 'ZdText',
              text: nameFunc,
            };
            cardItem.children?.push(textFunc);
            const results: IComponentRender = this.createTagResults();
            this.Gestores[nrFuncGestor].FUNCIONARIOS[nrFunc].NAOCONFORMIDADES.forEach((naoConformidadeItem: string, index: number) => {
              const naoConformidade: IComponentRender = this.createNaoConformidade(index.toString(), naoConformidadeItem);
              results.children?.push(naoConformidade);
            });
            cardItem.children?.push(results);
          });
          columnItem.children?.push(cardItem);
          rowComponent.children.push(columnItem);
        });
      })
      .catch((err: string) => {
        AlertService.show({
          name: 'basic-alert',
          top: true,
          left: false,
          color: 'error',
          text: err,
          timeout: 3000,
        });
      });
  }
}
