/* eslint-disable no-console */
import {
  Metadata, IDictionary, Router,
  IEventParam, Http, Loader,
} from '@zeedhi/core';
import {
  Col, Row, CollapseCard, Tabs, Button, TextInput, IComponentRender,
} from '@zeedhi/common';

export class GerenciarLocaisController {
  public localAtendimento = Loader.getInstance('LocalAtendimentoController');

  // Verifica qual o acesso e altera a exibição dos botões de "novo local" e "editar"
  public isVisibleElement: boolean = !(localStorage.getItem('idpermissslocais') === 'V');

  // Coleção para filtro
  public dataFilter: IDictionary = {
    inputLocalAt: '',
    inputLocal: '',
  };

  // Prototipo de CollapseCard para Agrupamento Completo
  public agCompletoCard(nm: string, nr: number, code: string, color: string): IDictionary {
    return {
      name: `${nr}`,
      component: 'ZdCollapseCard',
      lazyload: true,
      cssClass: 'zd-mb-3',
      events: {
        onCreated: '{{GerenciarLocaisController.setChildrenLocal}}',
      },
      header: [
        {
          name: 'row',
          component: 'ZdRow',
          cssClass: 'zd-ml-1',
          align: 'center',
          children: [
            {
              name: '',
              component: 'ZdText',
              text: 'mdi-office-building',
              tag: 'v-icon',
              isVisible: false,
            },
            {
              name: 'cardColor',
              component: 'ZdIcon',
              iconName: 'mdi-square',
              color: `${color}`,
            },
            {
              name: 'col0',
              component: 'ZdCol',
              cssClass: 'zd-mr-6',
              cols: 8,
              children: [
                {
                  name: `rowHeader${nr}`,
                  component: 'ZdRow',
                  align: 'center',
                  children: [
                    {
                      name: '',
                      component: 'ZdText',
                      text: nm,
                      cssClass: 'color-title-card margin',
                      style: {
                        'font-weight': 'bold',
                      },
                    },
                    {
                      name: `code${code}`,
                      component: 'ZdText',
                      cssClass: 'zd-mt-1 zd-ml-1',
                      text: `<span style="color: #6c757d; font: 14px Roboto, sans-serif">#${code}</span>`,
                      isVisible: true,
                    },
                  ],
                },
              ],
            },
            {
              name: 'col',
              component: 'ZdCol',
              cols: 3,
              cssClass: 'text-end zd-ml-6',
              children: [
                {
                  name: `add${nr}`,
                  component: 'ZdButton',
                  icon: true,
                  iconName: 'mdi-plus',
                  small: true,
                  cssClass: 'zd-mr-1',
                  events: {
                    click: '{{LocalAtendimentoController.setParentLocal}}',
                  },
                },
                {
                  name: 'addLocal',
                  component: 'ZdButton',
                  to: `/localAtendimento/${nr}`,
                  icon: true,
                  iconName: 'mdi-pencil',
                  small: true,
                  isVisible: true,
                },
              ],
            },
          ],
        },
      ],
      children: [
        {
          name: 'collapseDivider',
          component: 'ZdDivider',
          width: '96%',
          isVisible: false,
        },
      ],
    };
  }

  // Gera os Cards,Children de cada collapse card exibido na tela
  public agChildrenLocal(nm: string, nr: number, code: string) {
    return {
      name: `${nr}`,
      component: 'ZdCard',
      elevation: 2,
      tile: false,
      hover: true,
      cssClass: 'zd-mb-2', // to: `/agrupamentoSimples/${nr}`,
      children: [
        {
          name: `${nr}`,
          component: 'ZdRow',
          children: [
            {
              name: `${nm}`,
              component: 'ZdRow',
              align: 'center',
              events: {
                onCreated: '{{GerenciarLocaisController.setLocal}}',
              },
              children: [
                {
                  name: 'col',
                  component: 'ZdCol',
                  cols: 3,
                  cssClass: 'zd-ml-4',
                  children: [
                    {
                      name: `rowHeader${nr}`,
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'text',
                          component: 'ZdText',
                          text: `${nm}  `,
                          cssClass: 'sub-title-text-card',
                        },
                        {
                          name: 'text2',
                          component: 'ZdText',
                          cssClass: 'zd-ml-1',
                          text: `<span style="color: #6c757d; font: 13px Roboto, sans-serif">#${code}</span>`,
                        },
                      ],
                    },
                  ],
                },
              ],
            },
            {
              name: `add${nr}`,
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-plus',
              small: true,
              cssClass: 'zd-mr-1',
              events: {
                click: '{{LocalAtendimentoController.setParentLocal}}',
              },
            },
            {
              name: `edit${nr}`,
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-pencil',
              small: true,
              cssClass: 'zd-mr-1',
              to: `/localAtendimento/${nr}`,
            },
            {
              name: `btn${nr}`,
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-eye',
              small: true,
              events: {
                click: () => this.changeRouteAS(nr),
              },
            },
          ],
        },
      ],
    };
  }

  // Gera os filhos do Andar
  public local(nm: string, nr: number) {
    return {
      name: `text${nr}`,
      component: 'ZdText',
      cssClass: 'zd-ml-2 zd-mt-1 color-text-card',
      text: `${nm}; `,
    };
  }

  // Muda o label do button Novo local para Novo tipo de local, altera a rota e ajusta a margem
  public changeButton() {
    const tabObject = Metadata.getInstance<Tabs>('tabs');
    const buttonObject = Metadata.getInstance<Button>('button_newLocal');

    if (tabObject.activeTab === 0) {
      buttonObject.label = 'Novo Local';
      buttonObject.to = '/localAtendimento';
    }
    if (tabObject.activeTab === 1) {
      buttonObject.label = 'Novo Tipo de Local';
      buttonObject.to = '/tipoLocal/null';
    }
  }

  // Gera os Collapse/locaisAT do tipo agrupamento completo(Edificios) para a tab 'Locais'
  public setUnities() {
    const colObject = Metadata.getInstance<Col>('locals_col');
    colObject.children = [];

    Http.post('/getUnities', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        colObject.children.push(this.agCompletoCard(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE, dados.DSCODIGOLOCALCLI, dados.DSCORIDLOCAL) as IComponentRender);
      });
    });
  }

  // Gera os Cards/Children dos collapses
  public setChildrenLocal({ component }: IEventParam<CollapseCard>) {
    const params = {
      NRLOCALCLIENTEPAI: component.name,
    };
    Http.post('/getRowCard2', { params }).then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        component.addChild(< unknown > this.agChildrenLocal(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE, dados.DSCODIGOLOCALCLI) as IComponentRender);
      });
    });
  }

  // Gera os locaisAt localizados naquele local. Colunas dentro da row do card
  public setLocal({ component }: IEventParam<Row>) {
    const params = {
      NRLOCALCLIENTEPAI: component!.parent!.name,
    };
    Http.post('/getColCard', { params }).then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        component.addChild(this.local(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
      });
    });
  }

  // Procurar o pai correspondente
  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  // Filtro
  public filterInputLocalAt() {
    const tabObject = Metadata.getInstance<Tabs>('tabs');
    const searchObject = Metadata.getInstance<TextInput>('textSearch');

    if (tabObject.activeTab === 0) {
      this.dataFilter.inputLocalAt = searchObject.value;
      this.setUnities();
    } else {
      this.dataFilter.inputLocal = searchObject.value;
      this.setLocalType();
    }
  }

  // Chama a rota /agrupamentoSimples para o ID do agrupamento simples do card clicado
  public changeRouteAS(nr: number) {
    Router.replace(`/agrupamentoSimples/${nr}`);
  }

  /*  --------- Tab: Tipos de Local ----------  */

  public cardLocalType(nm: string, nr: number, code: number, tipo: string) {
    return {
      name: '',
      component: 'ZdCard',
      width: 'auto',
      cssClass: 'zd-mb-3',
      hover: true,
      children: [
        {
          name: `row${nr}`,
          component: 'ZdRow',
          children: [
            {
              name: '',
              component: 'ZdText',
              text: 'mdi-office-building',
              tag: 'v-icon',
              isVisible: false,
            },
            {
              name: '',
              component: 'ZdCol',
              cssClass: 'text-start',
              sm: 2,
              children: [
                {
                  name: '',
                  component: 'ZdText',
                  text: `${nm}`,
                  cssClass: 'color-title-card',
                  style: {
                    'font-weight': 'bold',
                  },
                },
              ],
            },
            {
              name: '',
              component: 'ZdCol',
              sm: 6,
              children: [
                {
                  name: '',
                  component: 'ZdText',
                  text: `${tipo}`,
                  style: {
                    color: '#6c757d',
                  },
                },
              ],
            },
            {
              name: '',
              component: 'ZdCol',
              sm: 3,
              cssClass: 'zd-ml-12 text-end',
              children: [
                {
                  name: `btn${nr}`,
                  component: 'ZdButton',
                  iconName: 'mdi-pencil',
                  icon: true,
                  small: true,
                  to: `/tipoLocal/${nr}`,
                  isVisible: this.isVisibleElement,
                },
              ],
            },
          ],
        },
      ],
    };
  }

  // requisição de Tipos de local
  public setLocalType() {
    const colObject = Metadata.getInstance<Col>('localTypesCol');
    colObject.children = [];
    Http.post('/getGLocalType', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        colObject.children.push(this.cardLocalType(dados.NMTIPOLOCALCLI, dados.NRTIPOLOCALCLI, dados.IDTIPOLOCALCLI, dados.TIPOLOCAL) as IComponentRender);
      });
    });
  }

  // Controlla a visibilidade do botão 'Novo Local'
  // public btnVisibility() {
  //   const buttonObject = Metadata.getInstance<Button>('button_newLocal');
  //   const idpermissslocais = localStorage.getItem('idpermissslocais');

  //   if (idpermissslocais === 'V') {
  //     buttonObject.isVisible = false;
  //   }
  // }

  // // Controlla a visibilidade do botão Editar da tab 'Tipos de Local'
  // public editBtnVisibility({ component }: IEventParam<Button>) {
  //   const idpermissslocais = localStorage.getItem('idpermissslocais');

  //   if (idpermissslocais === 'V') {
  //     component.isVisible = false;
  //   }
  // }
}
