import {
  Modal, ModalService, Form, Col, Icon,
} from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import ZdColorPicker from '@/components/colorPicker/ZdColorPicker';

export class ModalControllerAddAtividade {
  private headerFooterModal!: Modal;

  public async createHeaderFooterModal() {
    this.headerFooterModal = await ModalService.createFromJson('/modal/modalNovaAtividade', true);
  }

  public destroyHeaderFooterModal() {
    this.headerFooterModal.destroy();
  }

  public openHeaderFooterModal() {
    this.headerFooterModal.show();
  }

  public closeHeaderFooterModal() {
    this.headerFooterModal.hide();
    this.headerFooterModal.destroy();
    this.createHeaderFooterModal();
  }

  public changeColor() {
    const form = Metadata.getInstance<Form>('rowLinhaNomeTarefa');
    form.children[0].grid!.cols = '8';
    form.children[1].grid!.cols = '4';
    const col = form.getChildInstance<Col>('BlocoDeCor');
    let corAnterior = Metadata.getInstance<Icon>('SquareWithColor').color;
    corAnterior = corAnterior !== 'transparent' ? corAnterior : '#F44336FF';
    col.children = [
      {
        name: 'SelectedColor',
        component: 'ZdColorPicker',
        style: {
          position: 'relative',
          top: '-2em',
        },
        value: corAnterior,
      },
      {
        name: 'botaoConfirmaColor',
        component: 'ZdButton',
        label: 'Confirmar',
        style: {
          width: '100%',
          position: 'relative',
          top: '-1em',
        },
        events: {
          click: () => this.closeColor(),
        },
      },
    ];
  }

  public closeColor() {
    let color: any = Metadata.getInstance<ZdColorPicker>('SelectedColor');
    color = color.value;
    const form = Metadata.getInstance<Form>('rowLinhaNomeTarefa');
    form.children[0].grid!.cols = '10';
    form.children[1].grid!.cols = '2';
    const col = form.getChildInstance<Col>('BlocoDeCor');
    col.children = [
      {
        name: 'TituloColor',
        component: 'ZdText',
        text: 'Selecione uma cor:',
        grid: {
          cols: '12',
        },
      },
      {
        name: 'Color',
        component: 'ZdButton',
        iconName: 'mdi-palette',
        small: true,
        grid: {
          cols: '6',
        },
        events: {
          click: () => this.changeColor(),
        },
      },
      {
        name: 'SquareWithColor',
        component: 'ZdIcon',
        iconName: 'mdi-square',
        cssClass: 'zd-ml-3',
        color: `${color}`,
        grid: {
          cols: '6',
        },
        style: {
          border: '1px solid black',
          'margin-left': '10px !important',
        },
      },
    ];
  }
}
